﻿using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace LatLonToXYSample
{
    public partial class PlotALatitudeAndLongitudePointOnTheMap : UserControl
    {
        public PlotALatitudeAndLongitudePointOnTheMap()
        {
            InitializeComponent();
        }

        private void WpfMapLoaded(object sender, RoutedEventArgs e)
        {
           InitializeMap();
           combox.SelectedIndex = 0;
        }

        private void InitializeMap()
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;

            var layerOverlay = new LayerOverlay();
            wpfMap1.Overlays.Add(layerOverlay);

            var pointLayer = new InMemoryFeatureLayer();
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"..\..\Data\United States.png");
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var pointOverlay = new LayerOverlay();
            pointOverlay.TileType = TileType.SingleTile;
            pointOverlay.Layers.Add("PointLayer", pointLayer);
            wpfMap1.Overlays.Add("PointOverlay", pointOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);

            wpfMap1.Refresh();

        }

        private ShapeFileFeatureLayer GetShapeFileLayer()
        {
            var fLayer = new ShapeFileFeatureLayer(@"..\..\Data\oklahoma_highway.shp");
            fLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var color = new GeoColor(0,0,0);
            var lineStyle = new LineStyle
                                {
                                    OuterPen = new GeoPen(color, 1)
                                };
            fLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = lineStyle;
            return fLayer;
        }

        private ShapeFileFeatureLayer GetShapeFileLayer2()
        {
            //----------------------------------------------------------------------------------------------
            //save (wgs84) shapefile to new shapefile (nad27) then create and return nad27 shapefile layer
            //----------------------------------------------------------------------------------------------

            // From Projection and To Projection set.
            var proj4Projection = new ManagedProj4Projection();
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);

            string sourceShapeFile = @"..\..\Data\wgs84.shp";
            string targetShapeFile = @"..\..\Data\nad27.shp";
            ShapeFileFeatureLayer.SaveToProjection(sourceShapeFile, targetShapeFile, proj4Projection, OverwriteMode.Overwrite);

            //add new shapefile
            var fLayer = new ShapeFileFeatureLayer(@"..\..\Data\nad27.shp");
            fLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var color = new GeoColor(0, 0, 0);
            var lineStyle = new LineStyle
            {
                OuterPen = new GeoPen(color, 1)
            };
            fLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = lineStyle;
            return fLayer;
        }
        
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBoxItem lbItem = combox.SelectedItem as ListBoxItem;
            if (lbItem != null) MapCoordinates = lbItem.Content.ToString();


            //wgs84 lat/lon and equivalent x/y for State Plane Oklahoma North NAD27, feet
            double latitude = 35.5953650;
            double longitude = -99.6787521;
            double x = 1501009.66;
            double y = 221013.88;

            Feature feature;

            LayerOverlay pointOverlay = null;
            InMemoryFeatureLayer pointLayer = null;
            ManagedProj4Projection proj4Projection = new ManagedProj4Projection();

            if (MapCoordinates == "Lat/Lon")
            {
                feature = new Feature(longitude, latitude, "Point1");
                wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
                proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

                pointOverlay = (LayerOverlay)wpfMap1.Overlays["PointOverlay"];
                pointLayer = (InMemoryFeatureLayer)pointOverlay.Layers["PointLayer"];
                pointLayer.FeatureSource.Projection = proj4Projection;

                ShapeFileFeatureLayer shapeLayer = GetShapeFileLayer();

                if (!pointOverlay.Layers.Contains("LatLonShapeLayer"))
                {
                    pointOverlay.Layers.Add("LatLonShapeLayer", shapeLayer);
                }

                wpfMap1.CurrentExtent = new RectangleShape(-99.7175, 35.61735, -99.63797, 35.57408);
            }
            else 
            {
                // MapCoordinates == "X/Y"
                feature = new Feature(x, y, "Point1");
                wpfMap1.MapUnit = GeographyUnit.Feet;
                proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);
                proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);
                //proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

                pointOverlay = (LayerOverlay)wpfMap1.Overlays["PointOverlay"];
                pointLayer = (InMemoryFeatureLayer)pointOverlay.Layers["PointLayer"];
                pointLayer.FeatureSource.Projection = proj4Projection;

                //ShapeFileFeatureLayer shapeLayer = GetShapeFileLayer();
                ShapeFileFeatureLayer shapeLayer = GetShapeFileLayer2();

                if (!pointOverlay.Layers.Contains("XYShapeLayer"))
                {
                    pointOverlay.Layers.Add("XYShapeLayer", shapeLayer);
                }

                var upperLeftPointShape = new PointShape(x - 100000, y + 100000);
                var lowerRightPointShape = new PointShape(x + 100000, y - 100000);
                wpfMap1.CurrentExtent = new RectangleShape(upperLeftPointShape, lowerRightPointShape);
       
            }
            
            pointLayer.InternalFeatures.Clear();
            if (!pointLayer.InternalFeatures.Contains("Point1"))
            {
                pointLayer.InternalFeatures.Add("Point1", feature);
            }

            pointOverlay.Refresh();
            wpfMap1.Refresh();

        }

        public string MapCoordinates { get; set; }
    }
}
